"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportType = void 0;
const sass = __importStar(require("sass"));
const fibers_1 = __importDefault(require("fibers"));
// eslint-disable-next-line import/extensions
const dx_theme_builder_metadata_1 = require("../data/metadata/dx-theme-builder-metadata");
var ImportType;
(function (ImportType) {
    ImportType[ImportType["Index"] = 0] = "Index";
    ImportType[ImportType["Color"] = 1] = "Color";
    ImportType[ImportType["Unknown"] = 2] = "Unknown";
})(ImportType = exports.ImportType || (exports.ImportType = {}));
class Compiler {
    constructor() {
        this.changedVariables = {};
        this.importerCache = {};
        this.meta = dx_theme_builder_metadata_1.metadata;
        this.userItems = [];
    }
    compile(items, options) {
        this.changedVariables = {};
        this.userItems = items || [];
        let compilerOptions = {
            importer: this.setter.bind(this),
            functions: {
                'collector($map)': this.collector.bind(this),
            },
            fiber: fibers_1.default,
        };
        compilerOptions = Object.assign(Object.assign({}, compilerOptions), options);
        return new Promise((resolve, reject) => {
            sass.render(compilerOptions, (error, result) => {
                this.importerCache = {};
                if (error)
                    reject(error);
                else {
                    resolve({
                        result,
                        changedVariables: this.changedVariables,
                    });
                }
            });
        });
    }
    static getImportType(url) {
        if (url.endsWith('tb_index'))
            return ImportType.Index;
        if (url.startsWith('tb_'))
            return ImportType.Color;
        return ImportType.Unknown;
    }
    getMatchingUserItemsAsString(theme) {
        const meta = theme === 'generic' ? this.meta.generic : this.meta.material;
        const themeKeys = meta.map((item) => item.Key);
        return this.userItems
            .filter((item) => themeKeys.indexOf(item.key) >= 0)
            .map((item) => `${item.key}: ${item.value};`)
            .join('');
    }
    setter(url) {
        const importType = Compiler.getImportType(url);
        if (importType === ImportType.Unknown) {
            return null;
        }
        let content = this.importerCache[url];
        if (!content) {
            content = importType === ImportType.Index
                ? this.indexFileContent
                : this.getMatchingUserItemsAsString(url.replace('tb_', ''));
            this.importerCache[url] = content;
        }
        return { contents: content };
    }
    collector(map) {
        for (let mapIndex = 0; mapIndex < map.getLength(); mapIndex += 1) {
            const value = map.getValue(mapIndex);
            let variableValue;
            if (value instanceof sass.types.Color) {
                variableValue = `rgba(${value.getR()},${value.getG()},${value.getB()},${value.getA()})`;
            }
            else if (value instanceof sass.types.String) {
                variableValue = value.getValue();
            }
            else if (value instanceof sass.types.Number) {
                variableValue = `${value.getValue()}${value.getUnit()}`;
            }
            else if (value instanceof sass.types.List) {
                const listValues = [];
                for (let listIndex = 0; listIndex < value.getLength(); listIndex += 1) {
                    listValues.push(value.getValue(listIndex));
                }
                variableValue = listValues.join(value.getSeparator() ? ',' : ' ');
            }
            else {
                return sass.types.Null.NULL;
            }
            const variableKey = map.getKey(mapIndex).getValue();
            this.changedVariables[variableKey] = variableValue;
        }
        return sass.types.Null.NULL;
    }
}
exports.default = Compiler;
